"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const find_packages_1 = require("find-packages");
const semver = require("semver");
const majors = require("major-versions");
const commonTags = require("common-tags");
const chalk = require("chalk");
const path = require("path");
const symlink_dir_1 = require("symlink-dir");
const oneLine = commonTags.oneLine;
const highlight = chalk.yellow;
function default_1(root, opts) {
    return __awaiter(this, void 0, void 0, function* () {
        const pkgs = yield find_packages_1.default(root, {
            ignore: opts && opts.ignore
        });
        const pkgMap = createPkgMap(pkgs);
        Promise.all(Object.keys(pkgMap).map(name => linkDeps(pkgMap[name])));
        function linkDeps(pkg) {
            const dependencies = Object.assign({}, pkg.manifest.devDependencies, pkg.manifest.optionalDependencies, pkg.manifest.dependencies);
            Promise.all(Object.keys(dependencies)
                .filter(depName => pkgs.some(pkg => pkg.manifest.name === depName))
                .map(depName => {
                const range = dependencies[depName];
                const major = getMajorFromRange(range);
                return `${depName}@${major}`;
            })
                .map(pkgMajorId => pkgMap[pkgMajorId])
                .filter(Boolean)
                .filter(dependency => areCompatible(pkg.manifest.name, dependencies[dependency.manifest.name], dependency.manifest))
                .map(dependency => link(pkg, dependency)));
        }
    });
}
exports.default = default_1;
function getMajorFromRange(range) {
    const major = majors(range)[0];
    const index = major.indexOf('.');
    if (index === -1)
        return major;
    return major.substr(0, index);
}
function createPkgMap(pkgs) {
    const pkgMap = {};
    for (let pkg of pkgs) {
        pkgMap[createPkgMajorId(pkg.manifest.name, pkg.manifest.version)] = pkg;
    }
    return pkgMap;
}
function createPkgMajorId(name, version) {
    const major = semver.major(version);
    return `${name}@${major}`;
}
function areCompatible(dependentName, dependentRange, dependency) {
    if (semver.satisfies(dependency.version, dependentRange)) {
        return true;
    }
    const available = `${dependency.name}@${dependency.version}`;
    const needed = `${dependency.name}@${dependentRange}`;
    console.warn(oneLine `
    Local ${highlight(available)}
    cannot be used by ${highlight(dependentName)} which needs
    ${highlight(needed)}
  `);
    return false;
}
function link(pkg, dep) {
    return __awaiter(this, void 0, void 0, function* () {
        const target = path.join(pkg.path, 'node_modules', dep.manifest.name);
        yield symlink_dir_1.default(dep.path, target);
    });
}
//# sourceMappingURL=index.js.map